import argparse
from pathlib import Path

from tqdm import tqdm


def extract_para(path):
    fn = path / f"{path.name}.opf" / "base.txt"
    paras = ""
    with fn.open() as f_read:
        for line in f_read.readlines():
            if len(line) > 300:
                line = line.replace("{", "")
                line = line.replace("}", "")
                paras += line
    return paras


if __name__ == "__main__":
    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument("--path", type=str, help="directory path containing all the data")
    args = ap.parse_args()

    out_fn = Path("paragraphs.txt")
    for path in tqdm(list(Path(args.path).iterdir())):
        paras = extract_para(path)

        if paras:
            with out_fn.open("a") as f_append:
                f_append.write(paras)



import os

from openpecha.catalog.storage import GithubBucket

if __name__ == "__main__":

    GITHUB_BUCKET_CONFIG = {
        "catalog": {"start_id": 0, "end_id": 0},
        "token": os.environ.get("GITHUB_TOKEN"),
    }

    bucket = GithubBucket("OpenPecha", config=GITHUB_BUCKET_CONFIG)
    for pecha_id, base in bucket.get_all_pechas_base():
        print(pecha_id)
        for vol_base, vol_fn in base:
            print(vol_fn, vol_base)



import pytest
from pydantic import ValidationError

from openpecha.core.annotation import Span


def test_span_end_must_not_be_less_than_start():
    with pytest.raises(ValidationError):
        Span(start=2, end=1)



import tempfile

import pytest

from openpecha.catalog.manager import CatalogManager
from openpecha.formatters import *

metadata = {
    "source_metadata": {
        "title": "example-title",
        "subtitle": "v001",
        "author": "authors",
        "id": "2323",
    }
}


@pytest.mark.skip(reason="no urgent")
def test_googleocr():
    catalog = CatalogManager(
        formatter=GoogleOCRFormatter(), last_id_fn="ocr-machine-08_last_id"
    )
    catalog.ocr_to_opf("./tests/data/formatter/google_ocr/W00001")
    catalog.update_catalog()


@pytest.mark.skip(reason="no urgent")
def test_hfml_with_metadata():
    layers = ["Citation", "BookTitle", "Author"]
    catalog = CatalogManager(
        formatter=HFMLFormatter(output_path="./output", metadata=metadata),
        layers=layers,
    )
    catalog.add_hfml_item("./tests/data/formatter/hfml/P0001")
    catalog.update()


def get_fake_img():
    return tempfile.NamedTemporaryFile(delete=False, suffix=".png").name


@pytest.mark.skip(reason="creating github repo")
def test_create_emtpy_item():
    assets = {"image": [get_fake_img()]}
    catalog = CatalogManager(
        formatter=EmptyEbook(
            output_path="./output", metadata=metadata["source_metadata"], assets=assets
        ),
    )
    catalog.add_empty_item("this is text only")


if __name__ == "__main__":
    test_create_emtpy_item()



from collections import defaultdict

from openpecha.formatters import BaseFormatter


class TsadraFormatter(BaseFormatter):
    """
    OpenPecha Formatter for Tsadra DarmaCloud ebooks
    """

    def text_preprocess(self, text):
        """
        This is temporary method to remove all the critic markups and make existing
        markup consistent.
        """
        # remove critic markup
        text = text.replace("{++", "")
        text = text.replace("++}", "")

        # edit the existing markup
        text = text.replace("###", "#")  # book_title to title
        text = text.replace("##", "#")  # chapter_title to title
        text = text.replace("**", "`")  # change tsawa markup
        text = text.replace("~~", "~")  # change quote markup

        return text

    def format_layer(self, layers):
        """
        Post-processing for various layer to easily dump into yaml file. For eg, title annotation has only one char coord, #<title_text>
        yigchung annotations has part of char coord, *<yigchun_text>*.
        """
        for layer, anns in layers.items():
            out = {}
            anns = [e - 1 if i % 2 == 0 else e for i, e in enumerate(anns, start=1)]
            for i, e in enumerate(range(0, len(anns), 2)):
                out[i] = anns[e : e + 2]
            layers[layer] = out
        return layers

    def build_layers(self, text):
        """
        Parse all the layers annotation from the given text.
        """
        layers = defaultdict(list)
        i = 0
        is_title = False
        for c in text:
            if c == "#":
                layers["title"].append(i)
                is_title = True
            elif is_title and c == "\n":
                layers["title"].append(i)
                is_title = False
                i += 1
            elif c == "*":
                layers["yigchung"].append(i)
            elif c == "`":
                layers["tsawa"].append(i)
            elif c == "~":
                layers["quotes"].append(i)
            elif c == "[" or c == "]":
                layers["sapche"].append(i)
            else:
                i += 1

        return self.format_layer(layers)

    def get_base_text(self, m_text):
        m_text = m_text.replace("#", "")
        m_text = m_text.replace("*", "")
        m_text = m_text.replace("`", "")
        m_text = m_text.replace("~", "")
        m_text = m_text.replace("[", "")
        text = m_text.replace("]", "")
        return text


if __name__ == "__main__":
    formatter = TsadraFormatter("usage/new_layer_output")
    formatter.new_poti("usage/input/W1OP000002.txt")



from pathlib import Path

import pytest

from openpecha.serializers import EpubSerializer, HFMLSerializer, SerializeMd


# For testing read_base_layer, add_chars and apply_layer
@pytest.fixture(scope="module")
def opf_path():
    return Path("./tests/data/serialize/tsadra/P000100.opf")


# Test HFML Serializer
# hfml_opf_path = Path("tests/data/serialize_test/hfml/hfml.opf")


@pytest.mark.skip(reason="not important")
def test_hfml_serializer():

    opf_path = "./tests/data/serialize/hfml/P000002.opf/"
    output_path = "./output/P000002_hfml/"
    serializer = HFMLSerializer(opf_path)
    serializer.serialize(output_path=output_path)


def test_hfml_2_tsadra_serializer(opf_path):
    serializer = EpubSerializer(opf_path)
    serializer.apply_layers()
    out_fn = serializer.serialize()
    assert str(out_fn) == "output/epub_output/P000100.epub"



import argparse
import os
import shutil
from pathlib import Path

from git import Repo
from github import Github

g = Github(os.environ.get("GITHUB_TOKEN"))
org = g.get_organization("OpenPecha")


def populate_repo(repo_path, from_path):
    for o in from_path.iterdir():
        shutil.move(str(o), str(repo_path))
    from_path.rmdir()


def create_github_repo(path):
    repo = org.create_repo(path.name)
    repo.create_file("README.md", "add README.md", "", branch="master")
    return repo._ssh_url.value


def commit(repo, message):
    has_changed = False

    # add untrack files
    for file in repo.untracked_files:
        print(f"Added untracked file: {file}")
        if file:
            repo.git.add(file)
        if has_changed is False:
            has_changed = True

    # add modified files
    if repo.is_dirty() is True:
        for file in repo.git.diff(None, name_only=True).split("\n"):
            print(f"Added file: {file}")
            if file:
                repo.git.add(file)
            if has_changed is False:
                has_changed = True

    if has_changed is True:
        repo.git.commit("-m", message)
        repo.git.push("origin", "master")
        print("[INFO] Commited and Pushed ->", repo)


def create_repo_and_commit(path, message):
    if (path / ".git").is_dir():
        print(f"[INFO] updating repo: {path.name}")
        # update_repo(path)
        repo = Repo(path)
        commit(repo, message)
    else:
        print(f"[INFO] Cloning repo: {path.name}")
        backup_repo = path.parent / "repo_bake"
        path.rename(backup_repo)
        remote_repo_url = create_github_repo(path)
        repo = Repo.clone_from(remote_repo_url, path)
        populate_repo(path, backup_repo)
        commit(repo, message)


if __name__ == "__main__":

    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument("--path", type=str, help="directory path containing all the data")
    ap.add_argument("-m", type=str, help="git commit message")
    args = ap.parse_args()

    for path in Path(args.path).iterdir():
        create_repo_and_commit(path, args.m)



import re
import urllib

from diff_match_patch import diff_match_patch


class Format:
    def __init__(self):
        pass

    def apply_patch(self, patch, mode=None):
        if mode == "CM":
            return self.cm_format(patch)
        else:  # default mode. same as DMP natively does.
            return self.default_format(patch)

    @staticmethod
    def default_format(patch):
        op, data = patch
        if op == -1:
            return ""
        elif op == 1:
            return data
        elif op == 0:
            return data
        else:
            raise ValueError("wrong patch operation value")

    @staticmethod
    def cm_format(patch):
        """
        Formats individual patches in CM.
        As the formatting is applied one patch at a time, substitutions can't be
        calculated at this stage. it has to be generated once all patches have been applied
        (see self.cm_substitutions() )
        """
        op, data = patch
        if op == -1:
            return "{--" + data + "--}"
        elif op == 1:
            return "{++" + data + "++}"
        elif op == 0:
            return data
        else:
            raise ValueError("wrong patch operation value")

    @staticmethod
    def cm_substitutions(string):
        """
        merges all sequences of deletion+insertion into a substitution
        ( {-- xx--}{++ yy++} ==> {~~xx~>yy~~} )
        """
        if "--}{++" in string:
            return re.sub(
                r"{-- ([^{}\-+]+)--}{\+\+ ([^{}\-+]+)\+\+}", r"{~~\1~>\2~~}", string
            )
        else:
            return string


class FormattedDMP(diff_match_patch):
    def __init__(self):
        super().__init__()
        self.formatting = Format()

    def patch_apply(self, patches, text, mode=None):
        """Merge a set of patches onto the text.  Return a patched text, as well
        as a list of true/false values indicating which patches were applied.

        Args:
          patches: Array of Patch objects.
          text: Old text.
          mode: formatting mode (see Format class)

        Returns:
          Two element Array, containing the new text and an array of boolean values.
        """
        if not patches:
            return (text, [])

        # Deep copy the patches so that no changes are made to originals.
        patches = self.patch_deepCopy(patches)

        nullPadding = self.patch_addPadding(patches)
        text = nullPadding + text + nullPadding
        self.patch_splitMax(patches)

        # delta keeps track of the offset between the expected and actual location
        # of the previous patch.  If there are patches expected at positions 10 and
        # 20, but the first patch was found at 12, delta is 2 and the second patch
        # has an effective expected position of 22.
        delta = 0
        results = []
        for patch in patches:
            expected_loc = patch.start2 + delta
            text1 = self.diff_text1(patch.diffs)
            end_loc = -1
            if len(text1) > self.Match_MaxBits:
                # patch_splitMax will only provide an oversized pattern in the case of
                # a monster delete.
                start_loc = self.match_main(
                    text, text1[: self.Match_MaxBits], expected_loc
                )
                if start_loc != -1:
                    end_loc = self.match_main(
                        text,
                        text1[-self.Match_MaxBits :],
                        expected_loc + len(text1) - self.Match_MaxBits,
                    )
                    if end_loc == -1 or start_loc >= end_loc:
                        # Can't find valid trailing context.  Drop this patch.
                        start_loc = -1
            else:
                start_loc = self.match_main(text, text1, expected_loc)
            if start_loc == -1:
                # No match found.  :(
                results.append(False)
                # Subtract the delta for this failed patch from subsequent patches.
                delta -= patch.length2 - patch.length1
            else:
                # Found a match.  :)
                results.append(True)
                delta = start_loc - expected_loc
                if end_loc == -1:
                    text2 = text[start_loc : start_loc + len(text1)]
                else:
                    text2 = text[start_loc : end_loc + self.Match_MaxBits]
                if text1 == text2:
                    # Perfect match, just shove the replacement text in.
                    text = (
                        text[:start_loc]
                        + self.diff_text2(patch.diffs, mode)
                        + text[start_loc + len(text1) :]
                    )
                else:
                    # Imperfect match.
                    # Run a diff to get a framework of equivalent indices.
                    diffs = self.diff_main(text1, text2, False)
                    if (
                        len(text1) > self.Match_MaxBits
                        and self.diff_levenshtein(diffs) / float(len(text1))
                        > self.Patch_DeleteThreshold
                    ):
                        # The end points match, but the content is unacceptably bad.
                        results[-1] = False
                    else:
                        self.diff_cleanupSemanticLossless(diffs)
                        index1 = 0
                        for (op, data) in patch.diffs:
                            if op != self.DIFF_EQUAL:
                                index2 = self.diff_xIndex(diffs, index1)
                            if op == self.DIFF_INSERT:  # Insertion
                                text = (
                                    text[: start_loc + index2]
                                    + self.formatting.apply_patch((op, data), mode)
                                    + text[start_loc + index2 :]
                                )
                            elif op == self.DIFF_DELETE:  # Deletion
                                text = (
                                    text[: start_loc + index2]
                                    + self.formatting.apply_patch((op, data), mode)
                                    + text[
                                        start_loc
                                        + self.diff_xIndex(diffs, index1 + len(data)) :
                                    ]
                                )
                            if op != self.DIFF_DELETE:
                                index1 += len(data)
        # Strip the padding off.
        text = text[len(nullPadding) : -len(nullPadding)]
        if mode == "CM":
            text = self.formatting.cm_substitutions(text)
        return (text, results)

    def diff_text2(self, diffs, mode=None):
        """Compute and return the destination text (all equalities and insertions).

        Args:
          diffs: Array of diff tuples.
          mode: formatting mode. See Format class

        Returns:
          Destination text.
        """
        text = []
        for d in diffs:
            text.append(self.formatting.apply_patch(d, mode))
        return "".join(text)

    def format_patch(self, patch):
        """
        presents the given patch in a Critic Markup format
        :return: string
        """
        out = [str(patch).split("\n")[0] + "\n"]
        for p in patch.diffs:
            out.append(self.formatting.apply_patch(p, "CM"))
        return "".join(out)

    @staticmethod
    def decode_patch(patch):
        """
        decodes the content of a patch that diff-match-patch had encoded in %XX format.
        :param patch: the str repr of a patch_obj containing data encoded with urllib
        :type patch: str
        :return: the patch with readable content. \n are encoded to %0A to retain the patch's format
        :rtype: str
        """
        lines = patch.rstrip("\n").split("\n")
        out = []
        for line in lines:
            if not line.startswith("@"):
                line = line[0] + urllib.parse.unquote(line[1:]).replace("\n", "%0A")
            out.append(line)
        return "\n".join(out)


if __name__ == "__main__":
    orig = "zabcde"
    edited = "abbdey"

    dmp = FormattedDMP()
    patches = dmp.patch_make(orig, edited)

    applied = dmp.patch_apply(patches, orig)
    edit = dmp.patch_apply(patches, orig, mode="CM")

    print("orig:", orig)
    print("edited:", applied)
    print("edited == applied:", edited == applied[0])
    print("critic markup:", edit)



import shutil
from pathlib import Path
from typing import Dict, Union

from openpecha.core.layer import Layer, MetaData
from openpecha.utils import dump_yaml


class OpenPechaFS:
    def __init__(self, opf_path):
        self.base_path = "base"
        self.layers_path = "layers"
        self.pecha_id = None
        self._opf_path = opf_path
        self.output_dir = None

    def prepare_save(self, pecha_id, output_dir):
        self.pecha_id = pecha_id
        self.output_dir = output_dir
        self._opf_path = None

    @staticmethod
    def _mkdir(path: Path):
        path.mkdir(parents=True, exist_ok=True)
        return path

    @property
    def opf_path(self):
        if self._opf_path:
            return self._opf_path
        self._opf_path = self.output_dir / self.pecha_id / f"{self.pecha_id}.opf"
        return self._opf_path

    @property
    def meta_fn(self):
        return self.opf_path / "meta.yml"

    @property
    def index_fn(self):
        return self.opf_path / "index.yml"

    @property
    def assets_path(self):
        return self.opf_path / "assets"

    def save_meta(self, meta: MetaData):
        dump_yaml(eval(meta.json()), self.meta_fn)

    def save_base(self, base: Dict[str, str]):
        for basename, content in base.items():
            base_fn = self._mkdir(self.opf_path / self.base_path) / f"{basename}.txt"
            base_fn.write_text(content)

    def save_layers(self, layers: Dict[str, Layer]):
        for basename, base_layers in layers.items():
            for layername, layer in base_layers.items():
                layer_fn = (
                    self._mkdir(self.opf_path / self.layers_path / basename)
                    / f"{layername.value}.yml"
                )
                dump_yaml(eval(layer.json()), layer_fn)

    def save_index(self, index):
        if not index:
            return
        dump_yaml(index, self.index_fn)

    def save_assets(self, assets):
        for assets_type, content in assets.items():
            assets_type_dir = self.assets_path / assets_type
            assets_type_dir.mkdir(parents=True, exist_ok=True)
            for asset_fn in content:
                asset_fn = Path(asset_fn)
                dest_fn = assets_type_dir / asset_fn.name
                shutil.copyfile(str(asset_fn), str(dest_fn))


class OpenPecha:
    def __init__(
        self,
        base: Dict = {},
        layers: Dict[str, Layer] = {},
        index: Dict = {},
        meta: MetaData = None,
        opf_path: Path = None,
        assets: Dict = {},
    ):
        self._pecha_id = None
        self.base = base
        self.layers = layers
        self._meta = meta
        self.index = index
        self.assets = assets
        self.opfs = OpenPechaFS(opf_path)

    @property
    def id(self):
        if self._pecha_id:
            return self._pecha_id
        self._pecha_id = self.meta.id
        return self._pecha_id

    @property
    def meta(self):
        if self._meta:
            return self._meta
        self._meta = self.opf.read_meta()
        return self._meta

    def get_base(self, basename):
        if basename in self.base:
            return self.base[basename]
        self.base[basename] = self.opfs.get_base(basename)
        return self.base[basename]

    def get_layer(self, basename, layername):
        if basename in self.layers and layername in self.layers[basename]:
            return self.layers[basename][layername]

        self.base[basename] = self.opfs.get_base(basename)
        return self.base[basename]

    @classmethod
    def from_dir(cls, path: Union[str, Path]):
        return cls(opf_path=Path(path))

    def save(self, output_path: Union[str, Path]):
        self.opfs.prepare_save(self.id, Path(output_path))
        self.opfs.save_base(self.base)
        self.opfs.save_layers(self.layers)
        self.opfs.save_index(self.index)
        self.opfs.save_meta(self.meta)
        self.opfs.save_assets(self.assets)
        return self.opfs.opf_path



# -*- coding: utf-8 -*-

# Define here the models for your scraped items
#
# See documentation in:
# https://doc.scrapy.org/en/latest/topics/items.html
#
# Extracted data -> EbookItem -> Pipeline -> open-poti

import scrapy


class TsadraEbookItem(scrapy.Item):
    # Digital Publication Source
    org_bo = scrapy.Field()
    org_en = scrapy.Field()
    website = scrapy.Field()
    aquisition_data = scrapy.Field()

    # Crawled Metadata
    title_bo = scrapy.Field()
    title_en = scrapy.Field()
    collection_bo = scrapy.Field()
    collection_en = scrapy.Field()
    publisher_bo = scrapy.Field()
    publisher_en = scrapy.Field()
    author_bo = scrapy.Field()
    author_en = scrapy.Field()

    sku = scrapy.Field()
    category_bo = scrapy.Field()
    category_en = scrapy.Field()

    description_bo = scrapy.Field()
    description_en = scrapy.Field()

    filename = scrapy.Field()



import argparse
import datetime
import os
from pathlib import Path

from github import Github

g = Github(os.environ.get("GITHUB_TOKEN"))
org = g.get_organization("OpenPecha")


if __name__ == "__main__":
    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument("--prefix", "-p", type=str, default="P", help="Repo prefix")
    ap.add_argument("--start", "-s", type=int, help="Start pecha number")
    ap.add_argument("--end", "-e", type=int, help="End pecha number")
    args = ap.parse_args()

    delete_from_date = datetime.datetime(2020, 3, 23, 15, 0, 4)

    if args.start and args.end is None:
        repo_name = f"{args.prefix}{args.start:06}"
        repo = org.get_repo(repo_name)
        repo.delete()
        print(f"[INFO] {repo_name} deleted")
    elif args.start and args.end:
        for repo in org.get_repos("all"):
            if repo.name.startswith(args.prefix):
                repo_num = int(repo.name[len(args.prefix) :])
                if args.start <= repo_num >= args.end:
                    repo.delete()
                    print(f"[INFO] {repo} deleted.")

            # else:
            #     if repo.created_at > delete_from_date:
            #         repo.delete()
            #         print(
            #             f"[INFO] Deleting {repo.name} created at {str(repo.created_at)}"
            #         )



import pytest
from pydantic import ValidationError

from openpecha.core.layer import Layer, LayerEnum


def test_layer_model():
    layer = Layer(
        annotation_type=LayerEnum.book_title, revision="00001", annotations={}
    )
    assert layer.annotation_type.value == "BookTitle"
    assert layer.revision == "00001"


def test_not_supported_layer():
    with pytest.raises(ValidationError):
        Layer(annotation_type="NotSupportedLayer", revision="00001", annotations={})


def test_revision_should_be_int_parsible():
    with pytest.raises(ValidationError):
        Layer(annotation_type=LayerEnum.book_title, revision="1aaa", annotations={})



import datetime
from pathlib import Path

import requests
import scrapy
from inline_requests import inline_requests

from bo_crawler.items import TsadraEbookItem


class TsadraSpider(scrapy.Spider):
    name = "tsadra"
    start_urls = ["http://dharmacloud.tsadra.org/library/"]

    def __init__(self):
        self.path = Path("../data/tsadra/data/ebooks")
        self.path.mkdir(parents=True, exist_ok=True)

    def create_item(self):
        item = TsadraEbookItem()
        item = self._init_item(item)
        return item

    def _init_item(self, item):
        item["org_bo"] = ""
        item["org_en"] = ""
        item["website"] = ""
        item["aquisition_data"] = ""
        item["title_bo"] = ""
        item["title_en"] = ""
        item["collection_bo"] = ""
        item["collection_en"] = ""
        item["publisher_bo"] = ""
        item["publisher_en"] = ""
        item["author_bo"] = ""
        item["author_en"] = ""
        item["sku"] = ""
        item["category_bo"] = ""
        item["category_en"] = ""
        item["description_bo"] = ""
        item["description_en"] = ""
        item["filename"] = ""
        return item

    def parse(self, response):
        ebook_page_urls = response.css(".product-images::attr(href)").extract()
        for ebook_page_url in ebook_page_urls:
            yield scrapy.Request(url=ebook_page_url, callback=self.download_ebook)

        # follow paginataion links
        next_page = response.css("a.next::attr(href)").extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)

    @inline_requests
    def download_ebook(self, response):
        title_bo = response.css("h1.product_title::text").extract_first()
        print(title_bo)

        # metadata
        raw_metadata_en, tib_page_url = self.parse_metadata(response, lang="en")
        if raw_metadata_en:
            item = self.create_item()
            item = self.get_metadata(raw_metadata_en, item, lang="en")

            # some page doesn't have Tibetan version
            if tib_page_url:
                tib_page_response = yield scrapy.Request(tib_page_url)
                raw_metadata_bo = self.parse_metadata(tib_page_response, lang="bo")
                item = self.get_metadata(raw_metadata_bo, item, lang="bo")

            item["org_bo"] = "མཐའ་ཡས་དཔེ་མཛོད།"
            item["org_en"] = "Timeless Treasuries"
            item["website"] = "http://dharmacloud.tsadra.org"
            item["aquisition_data"] = str(datetime.date.today())
            item["title_bo"] = title_bo

            # download the ebook
            dl_link = response.css("a.download_button::attr(href)").extract_first()
            ebook_filename = dl_link.split("/")[-1]
            item["filename"] = ebook_filename
            target = (self.path / f"{ebook_filename}").resolve()
            if not target.is_file():
                target.write_bytes(requests.get(dl_link).content)
                print("******* Download Completed !!! ********")

            yield item

    def parse_metadata(self, response, lang):
        product_details = response.css(
            "div.woocommerce-product-details__short-description *::text"
        ).extract()
        product_meta = response.css("div.product_meta *::text").extract()
        des = response.css("div#tab-description *::text").extract()
        if lang == "en" and "DOWNLOAD EBOOK" in product_details:
            tib_page_url = response.css("a[title=Tibetan]::attr(href)").get()
            return product_details[:-2] + product_meta + des, tib_page_url
        elif lang == "bo":
            return product_details[:-2] + product_meta + des
        else:
            return "", ""

    def get_metadata(self, raw_data, item, lang):
        metadata_key = {
            "collection": {"en": "Collection:", "bo": "ཕྱོགས་བསྡུས།"},
            "title": {"en": "Title:"},
            "author": {"en": "Author:", "bo": "མཛད་པ་པོ།"},
            "publisher": {"en": "Publisher:", "bo": "པར་འགྲེམས་བྱེད་པོ།"},
            "sku": {"en": "SKU:", "bo": "SKU:"},
            "category": {"en": "Category:", "bo": "Category:"},
            "description": {"en": "Description", "bo": "Description"},
        }

        md_type = ""

        for line in raw_data:
            line = line.strip()
            line = line.replace("\xa0", " ")
            # pdb.set_trace()
            if line:
                if line == metadata_key["collection"][lang]:
                    md_type = "collection"
                elif lang == "en" and metadata_key["title"][lang] in line:
                    md_type = "title"
                elif line == metadata_key["author"][lang] or (
                    lang == "en" and line == "Authors:"
                ):
                    md_type = "author"
                elif line == metadata_key["publisher"][lang]:
                    md_type = "publisher"
                elif line == metadata_key["sku"][lang]:
                    md_type = "sku"
                elif line == metadata_key["category"][lang]:
                    md_type = "category"
                elif line == metadata_key["description"][lang]:
                    md_type = "description"

                if md_type == "collection" and line != metadata_key["collection"][lang]:
                    item[f"collection_{lang}"] += line
                elif md_type == "title":
                    title_values = line.split(":")[1:]
                    print("+++++++++++++++++++++++", line, title_values)
                    if len(title_values) > 1:
                        item[f"title_{lang}"] += ":".join(
                            [value.strip() for value in title_values]
                        )
                    else:
                        item[f"title_{lang}"] += title_values.pop().strip()
                elif md_type == "author" and line != metadata_key["author"][lang]:
                    item[f"author_{lang}"] += line
                elif md_type == "publisher" and line != metadata_key["publisher"][lang]:
                    item[f"publisher_{lang}"] += line
                elif md_type == "sku" and line != metadata_key["sku"][lang]:
                    item["sku"] += line
                elif md_type == "category" and line != metadata_key["category"][lang]:
                    item[f"category_{lang}"] += line
                elif (
                    md_type == "description"
                    and line != metadata_key["description"][lang]
                ):
                    item[f"description_{lang}"] += line

        return item



from pydantic import BaseModel, validator


class Span(BaseModel):
    start: int
    end: int

    @validator("*")
    def span_must_not_be_neg(cls, v):
        if v < 0:
            raise ValueError("span shouldn't be negative")
        return v

    @validator("end")
    def end_must_not_be_less_than_start(cls, v, values, **kwargs):
        if "start" in values and v < values["start"]:
            raise ValueError("Span end must not be less than start")
        return v


class AnnBase(BaseModel):
    span: Span



from pathlib import Path

from openpecha.formatters import HFMLFormatter, TsadraFormatter
from openpecha.serializers import EpubSerializer, HFMLSerializer

if __name__ == "__main__":

    # ebook_path = "./output/demo/src/P000111/OEBPS/"
    # opfs_path = "./output/demo/output"
    # opf_path = "./output/demo/output/P000111/P000111.opf/"
    # hfml_path = "./output/demo/output/P000111_hfml/"
    # ebook_output_path = "./output/demo/output/ebooks"
    # pecha_id = 111
    # pecha_name = f"P{pecha_id:06}"

    ebook_path = "./output/demo/src/tsadra_publication/RDI-TOK-06-1/OEBPS/"
    opfs_path = "./output/demo/output"
    # opf_path = "./output/demo/output/P000101/P000101.opf/"
    opf_path = "./output/demo/output/P008259/P008259.opf/"
    hfml_path = "./output/demo/output/P000101_hfml/"
    ebook_output_path = "./output/demo/output/ebooks"
    pecha_id = 101
    pecha_name = f"P{pecha_id:06}"

    # 1. Format Tsadra Ebook to OPF (OpenPecha Format)
    # formatter = TsadraFormatter(output_path=opfs_path)
    # formatter.create_opf(ebook_path, pecha_id)

    # # 2. Serialize OPF to HFML (Human Friendly Markup Language)
    # serializer = HFMLSerializer(opf_path)
    # serializer.serialize(output_path=hfml_path)

    # # 3. Format HFML to OPF
    # formatter = HFMLFormatter(output_path=opfs_path)
    # formatter.create_opf(f"{hfml_path}/{pecha_name}", pecha_id)

    # 4. Convert OPF to Ebook
    serializer = EpubSerializer(Path(opf_path))
    serializer.apply_layers()
    serializer.serialize(ebook_output_path)



import json
from pathlib import Path

import pytest

from openpecha.formatters import GoogleOCRFormatter, HFMLFormatter, TsadraFormatter
from openpecha.formatters.formatter import LocalIdManager
from openpecha.formatters.layers import AnnType


class TestHFMLFormatter:
    def test_get_base_text(self):
        m_text = Path("tests/data/formatter/hfml/kangyur_01.txt").read_text()
        formatter = HFMLFormatter()

        text = formatter.text_preprocess(m_text)
        formatter.build_layers(text, len([text]))
        result = formatter.get_base_text()

        expected = Path("tests/data/formatter/hfml/kangyur_base.txt").read_text()

        assert result == expected

    def test_build_layers(self):
        m_text1 = Path("tests/data/formatter/hfml/kangyur_01.txt").read_text()
        m_text2 = Path("tests/data/formatter/hfml/kangyur_02.txt").read_text()
        m_text3 = Path("tests/data/formatter/hfml/kangyur_03.txt").read_text()
        formatter = HFMLFormatter()

        text1 = formatter.text_preprocess(m_text1)
        text2 = formatter.text_preprocess(m_text2)
        text3 = formatter.text_preprocess(m_text3)
        texts = [text1, text2, text3]
        for text in texts:
            result = formatter.build_layers(text, len(texts))

        result = formatter.get_result()
        expected_result = {
            AnnType.book_title: [[], [], []],
            AnnType.book_number: [[], [], []],
            AnnType.author: [[], [], []],
            AnnType.poti_title: [
                [(None, {"span": {"start": 0, "end": 24}})],
                [(None, {"span": {"start": 0, "end": 24}})],
                [(None, {"span": {"start": 0, "end": 24}})],
            ],
            AnnType.chapter: [[(None, {"span": {"start": 98, "end": 125}})], [], []],
            AnnType.citation: [
                [],
                [
                    (1000020, {"span": {"start": 164, "end": 202}}),
                    (1000021, {"span": {"start": 204, "end": 241}}),
                ],
                [(1000024, {"span": {"start": 97, "end": 162}})],
            ],
            AnnType.pagination: [
                [
                    (
                        1000000,
                        {
                            "page_index": "1a",
                            "page_info": "kk",
                            "reference": None,
                            "span": {"start": 0, "end": 24},
                        },
                    ),
                    (
                        1000001,
                        {
                            "page_index": "1b",
                            "page_info": "kl",
                            "reference": None,
                            "span": {"start": 27, "end": 676},
                        },
                    ),
                    (
                        1000027,
                        {
                            "page_index": "2a",
                            "page_info": "lm",
                            "reference": None,
                            "span": {"start": 679, "end": 2173},
                        },
                    ),
                ],
                [
                    (
                        1000015,
                        {
                            "page_index": "1a",
                            "page_info": "kk",
                            "reference": None,
                            "span": {"start": 0, "end": 0},
                        },
                    ),
                    (
                        1000016,
                        {
                            "page_index": "1b",
                            "page_info": "",
                            "reference": None,
                            "span": {"start": 0, "end": 266},
                        },
                    ),
                ],
                [
                    (
                        1000022,
                        {
                            "page_index": "1a",
                            "page_info": "ko",
                            "reference": None,
                            "span": {"start": 0, "end": 266},
                        },
                    )
                ],
            ],
            AnnType.topic: [
                [
                    (
                        1000002,
                        {"work_id": "T1", "span": {"vol": 1, "start": 27, "end": 2046}},
                    )
                ],
                [
                    (
                        1000014,
                        {
                            "work_id": "t2",
                            "span": {"vol": 1, "start": 2046, "end": 2173},
                        },
                    )
                ],
                [
                    (
                        1000017,
                        {"work_id": "T2", "span": {"vol": 2, "start": 26, "end": 266}},
                    )
                ],
                [
                    (
                        1000023,
                        {"work_id": "T3", "span": {"vol": 3, "start": 26, "end": 243}},
                    )
                ],
                [
                    (
                        1000026,
                        {"work_id": "t4", "span": {"vol": 3, "start": 243, "end": 266}},
                    )
                ],
            ],
            AnnType.sub_topic: [
                [
                    [
                        (
                            1000003,
                            {
                                "work_id": "T1-1",
                                "span": {"vol": 1, "start": 27, "end": 1352},
                            },
                        )
                    ],
                    [
                        (
                            1000005,
                            {
                                "work_id": "T1-2",
                                "span": {"vol": 1, "start": 1352, "end": 1496},
                            },
                        )
                    ],
                    [
                        (
                            1000006,
                            {
                                "work_id": "T1-6",
                                "span": {"vol": 1, "start": 1496, "end": 2046},
                            },
                        )
                    ],
                ],
                [[]],
                [
                    [
                        (
                            1000018,
                            {
                                "work_id": "T1-8",
                                "span": {"vol": 2, "start": 26, "end": 140},
                            },
                        )
                    ],
                    [
                        (
                            1000019,
                            {
                                "work_id": "T1-9",
                                "span": {"vol": 2, "start": 140, "end": 266},
                            },
                        )
                    ],
                ],
                [[]],
                [[]],
            ],
            AnnType.sabche: [
                [(1000008, {"span": {"start": 1548, "end": 1936}})],
                [],
                [],
            ],
            AnnType.tsawa: [[(1000004, {"span": {"start": 420, "end": 739}})], [], []],
            AnnType.yigchung: [
                [],
                [],
                [(1000025, {"span": {"start": 164, "end": 241}})],
            ],
            AnnType.correction: [
                [
                    (
                        1000010,
                        {"correction": "མཆིའོ་", "span": {"start": 1838, "end": 1843}},
                    )
                ],
                [],
                [],
            ],
            AnnType.error_candidate: [
                [
                    (1000012, {"span": {"start": 2040, "end": 2042}}),
                    (1000013, {"span": {"start": 2044, "end": 2045}}),
                ],
                [],
                [],
            ],
            AnnType.peydurma: [
                [
                    (1000007, {"span": {"start": 1518, "end": 1518}}),
                    (1000009, {"span": {"start": 1624, "end": 1624}}),
                    (1000011, {"span": {"start": 1938, "end": 1938}}),
                ],
                [],
                [],
            ],
            AnnType.archaic: [[], [], []],
            AnnType.durchen: [[], [], []],
        }

        for layer in result:
            assert result[layer] == expected_result[layer]

    def test_tofu_id(self):
        formatter = HFMLFormatter()
        formatter.dirs = {}
        formatter.dirs["layers_path"] = Path("tests/data/formatter/hfml/tofu-id")
        layers = [layer.stem for layer in formatter.dirs["layers_path"].iterdir()]
        old_layers = formatter.get_old_layers(layers)
        local_id2uuid = LocalIdManager(old_layers)
        local_id2uuid.add("tsawa", "v001", 1231232)


class TestGoogleOCRFormatter:
    @pytest.fixture(scope="class")
    def get_resources(self):
        data_path = Path("tests/data/formatter/google_ocr/W0001/v001")
        responses = [
            (json.load(fn.open()), fn.stem)
            for fn in sorted(list((data_path).iterdir()))
        ]
        formatter = GoogleOCRFormatter()
        return formatter, data_path, responses

    def test_get_base_text(self, get_resources):
        formatter, data_path, responses = get_resources
        formatter.build_layers(responses, "")

        result = formatter.get_base_text()

        # TODO: create base-text
        # expected = (data_path/'v001.txt').read_text()
        # assert result == expected

    def test_build_layers(self, get_resources):
        formatter, data_path, responses = get_resources

        result = formatter.build_layers(responses, "")

        expected = {"pages": [(0, 19), (24, 888), (893, 1607), (1612, 1809)]}

        for result_page, expected_page in zip(result["pages"], expected["pages"]):
            assert result_page[:2] == expected_page


class TestTsadraFormatter:
    def test_tsadra_formatter(self):
        m_text_01 = Path("tests/data/formatter/tsadra/htmls/cover.xhtml").read_text()
        m_text_02 = Path(
            "tests/data/formatter/tsadra/htmls/tsadra_02.xhtml"
        ).read_text()
        m_texts = [m_text_01, m_text_02]
        formatter = TsadraFormatter()
        for m_text in m_texts:
            text = formatter.text_preprocess(m_text)
            formatter.build_layers(text)
        result = formatter.get_result()

        expected_result = {
            AnnType.book_title: [
                [(None, {"iscover": True, "span": {"start": 0, "end": 84}})]
            ],
            AnnType.book_number: [[]],
            AnnType.poti_title: [[]],
            AnnType.author: [
                [
                    (None, {"span": {"start": 86, "end": 109}}),
                    (None, {"span": {"start": 111, "end": 134}}),
                    (None, {"span": {"start": 136, "end": 181}}),
                ]
            ],
            AnnType.chapter: [[(None, {"span": {"start": 183, "end": 200}})]],
            AnnType.topic: [[]],
            AnnType.sub_topic: [[]],
            AnnType.pagination: [[]],
            AnnType.tsawa: [
                [
                    (None, {"span": {"start": 4150, "end": 4300}, "isverse": True}),
                    (None, {"span": {"start": 5122, "end": 5298}, "isverse": True}),
                ]
            ],
            AnnType.citation: [
                [
                    (None, {"span": {"start": 3993, "end": 4132}, "isverse": False}),
                    (None, {"span": {"start": 4302, "end": 4418}, "isverse": True}),
                ]
            ],
            AnnType.sabche: [
                [
                    (None, {"span": {"start": 5091, "end": 5120}}),
                    (None, {"span": {"start": 7313, "end": 7375}}),
                ]
            ],
            AnnType.yigchung: [[(None, {"span": {"start": 7273, "end": 7311}})]],
        }

        for layer in result:
            print(result[layer])
            assert result[layer] == expected_result[layer]

    def test_tsadra_get_base_text(self):
        m_text1 = Path("tests/data/formatter/tsadra/htmls/cover.xhtml").read_text()
        m_text2 = Path("tests/data/formatter/tsadra/htmls/tsadra_02.xhtml").read_text()
        texts = [m_text1, m_text2]
        formatter = TsadraFormatter()
        for text in texts:
            text = formatter.text_preprocess(text)
            formatter.build_layers(text)
        result = formatter.get_base_text()
        expected1 = Path("tests/data/formatter/tsadra/tsadra_base1.txt").read_text()
        assert result == expected1


if __name__ == "__main__":
    # TestHFMLFormatter().test_tofu_id()
    # path = Path("./tests/data/formatter/tsadra_hfml/")
    # path = Path("./output/tsadra_hfml/tsadra_hfml.opf/")

    # path = "./output/chagchen/"
    # pecha_id = 9
    # formatter = HFMLFormatter()
    # formatter.create_opf(path, pecha_id)

    # path = "./output/demo/src/P000101/OEBPS/"
    # formatter = TsadraFormatter()
    # formatter.create_opf(path, 9)
    TestTsadraFormatter().test_tsadra_formatter()



"""Create works in OpenPecha."""
import xml.etree.ElementTree as ET
from pathlib import Path
from xml.dom import minidom

import requests
import yaml
from pyewts import pyewts


class config:
    """Config class of the module."""

    resources_path = Path("resources/works")
    bdrc_outlines_path = resources_path / "db" / "tbrc" / "tbrc-outlines"
    work_fn = resources_path / "W00000001.yml"
    work_output = Path("../works/")
    converter = pyewts()


class Work:
    """Class for Work which contain all its the instances."""

    def __init__(self, work_fn=None):
        self.work_fn = Path(work_fn) if work_fn else work_fn
        self.work = self._create_or_load()

    def __repr__(self):
        return f"Work(title={self.work['title']}, author={self.work['author']}, bdrc-work-id={self.work['bdrc-work-id']}, n_instances={len(self.work['instances'])})"

    def _create_or_load(self):
        if not self.work_fn:
            return {
                "title": "",
                "author": "",
                "bdrc-work-id": "",
                "wikidata-id": "",
                "instances": [],
            }
        return yaml.safe_load(self.work_fn.read_text())

    def save(self, fn):
        self.work_fn = fn
        yaml.dump(
            self.work,
            self.work_fn.open("w"),
            default_flow_style=False,
            sort_keys=False,
            allow_unicode=True,
        )
        print(f"[INFO] work is save at: {self.work_fn}")

    def _to_rid(self, work_id):
        """Return rid version of work_id."""
        return work_id[0] + work_id[2:]

    def _get_title(self, rid):
        """Return title for instances with `rid`."""
        bdrc_metadata_url = f"https://www.tbrc.org/xmldoc?rid={rid}"
        r = requests.get(bdrc_metadata_url)
        try:
            root = ET.fromstring(r.content.decode("utf-8"))
        except Exception:
            return ""
        title_tag = root[0]
        return config.converter.toUnicode(title_tag.text)

    def _get_instances_id(self, fn):
        for work_id in fn.read_text().split("\n"):
            yield work_id.strip()

    def create_instance(self, title, rid, g_ocr=None, n_ocr=None, trans=None):
        instance = {title: {}}
        for k, v in [
            ("bdrc-instance-id", rid),
            ("google-ocr", g_ocr),
            ("namsel-ocr", n_ocr),
            ("transcription", trans),
        ]:
            if not v:
                continue
            instance[title][k] = v
        return instance

    def add_instances(self, instances_id_fn):
        for inst_id in self._get_instances_id(instances_id_fn):
            print(f"\t- adding isntance {inst_id}")
            title = self._get_title(inst_id)
            instance = self.create_instance(title, inst_id)
            self.work["instances"].append(instance)


def clean_text_id(text_id):
    if not text_id:
        return ""
    cats = "  abcd"
    text_id = text_id.strip()
    if text_id == "00":
        return ""
    elif "-" in text_id:
        id_, cat = text_id.split("-")
        if not cat or len(cat) > 1:
            return ""
        return f"{id_}{cats[int(cat.strip())]}"
    elif ";" in text_id:
        return text_id.split(";")[0].strip()
    else:
        return f"D{text_id}"


def get_value(parent, child, attr, convert=False):
    for node in parent.getElementsByTagName(f"outline:{child}"):
        attrs = node.attributes
        if "type" in attrs and attrs["type"].value == attr:
            childnodes = node.childNodes
            if not childnodes:
                return ""
            value = childnodes[0].data
            if convert:
                return config.converter.toUnicode(value)
            return value
    return ""


def get_works_from_bdrc_outlines(fn):
    dom = minidom.parse(str(fn))
    for node in dom.getElementsByTagName("outline:node"):
        if node.attributes["type"].value != "text":
            continue
        bdrc_work_id = node.attributes["RID"].value
        title = get_value(node, "title", "bibliographicalTitle", convert=True)
        author = get_value(node, "creator", "hasMainAuthor", convert=True)
        # toh = get_value(node, "description", "toh")
        # bdrc_work_id = clean_text_id(toh)
        if title or author or bdrc_work_id:
            work = Work()
            work.work["title"] = title
            work.work["author"] = author
            work.work["bdrc-work-id"] = bdrc_work_id
            yield work


if __name__ == "__main__":
    for i, work in enumerate(
        get_works_from_bdrc_outlines(config.bdrc_outlines_path / "O2MS16391.xml"),
        start=4,
    ):
        work_fn = config.work_output / f"W{i:08}.yml"
        print(work)
        work.save(work_fn)



"""Script to update the pecha mssing in catalog and sort the catalog."""

import argparse
import os
from pathlib import Path

import requests
import yaml
from github import Github


def to_csv_format(rows, cols_name):
    """Return csv formated data."""
    data = [cols_name] + rows
    lines = [",".join([o for o in line]) for line in data]
    return "\n".join(lines)


def get_file_content(url, in_bytes=False):
    """Download file on given `url` and returns in bytes or text with `in_bytes`."""
    r = requests.get(url)
    if r.status_code != 200:
        return
    content = r.content
    if in_bytes:
        return content
    return content.decode("utf-8").strip()


def sort_catalog(data):
    """Sort catalog base on pecha number order."""
    return sorted(data, key=lambda x: x[0])


def get_catalog():
    """Return sorted openpecha catalog."""
    catalog_csv_url = "https://raw.githubusercontent.com/OpenPecha/openpecha-catalog/master/data/catalog.csv"
    lines = get_file_content(catalog_csv_url).split("\n")
    catalog_data = [line.split(",") for line in lines]
    cols_name = catalog_data.pop(0)
    sorted_catalog_data = sort_catalog(catalog_data)
    return cols_name, sorted_catalog_data


def to_pecha_id_link(pecha_id):
    """Return pecha_id_link for `pecha_id`."""
    return f"[{pecha_id}](https://github.com/OpenPecha/{pecha_id})"


def get_pecha_metadata(pecha_id):
    """Return metadata in tuple for given `pecha_id."""
    metadata_url = f"https://raw.githubusercontent.com/OpenPecha/{pecha_id}/master/{pecha_id}.opf/meta.yml"
    content = get_file_content(metadata_url, in_bytes=True)
    if not content:
        return
    metadata_dict = yaml.safe_load(content)
    csv_row = [
        to_pecha_id_link(pecha_id),
        metadata_dict["source_metadata"]["title"],
        "",
        "",
        metadata_dict["source_metadata"]["id"],
    ]
    return csv_row


def get_pecha_num_list(pechas):
    """Extract all the pecha numbers from `catalog_data.

    Args:
        pechas (list): list of pecha metatdata in list.

    Returns:
        list: list of pecha numbers.
    """
    return [int(pecha[0][2:8]) for pecha in pechas]


def update_catalog(start):
    """Update the catalog with missing pecha.

    Args:
        start (int): starting pecha number.

    Returns:
        str: updated catalog data in csv format.
    """
    cols_name, pechas = get_catalog()
    pecha_nums = get_pecha_num_list(pechas)
    missing_pechas = []
    for num in range(start, max(pecha_nums) + 1):
        if num in pecha_nums:
            continue
        pecha_metadata = get_pecha_metadata(f"P{num:06}")
        if pecha_metadata:
            print(f"[INFO] adding {pecha_metadata[0][1:8]}")
            missing_pechas.append(pecha_metadata)
    updated_catalog = pechas + missing_pechas
    updated_catalog = sort_catalog(updated_catalog)
    return to_csv_format(updated_catalog, cols_name)


if __name__ == "__main__":
    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument("--start", "-s", type=int, help="start pecha number")
    ap.add_argument("--debug", action="store_true")
    args = ap.parse_args()
    output_fn = Path("resources/works/catalog.csv")
    cols_name, catalog_data = get_catalog()
    if args.debug:
        print(to_csv_format(catalog_data, cols_name))
    output_fn.write_text(update_catalog(args.start))



import os
from collections import defaultdict
from pathlib import Path

import yaml

from openpecha.serializers import SerializeFootNote

error_logs_fn = Path("./error_logs") / Path(__file__).stem
if error_logs_fn.is_file():
    error_logs_fn.unlink()


def to_bo_pagination(text):
    def index_to_bo(index):
        chars = list(index)
        result = ""
        for c in chars:
            if c.isdigit():
                result += bo_num[int(c)]
            elif c == "a":
                result += sides[0]
            else:
                result += sides[1]
        return result + "་"

    bo_num = "༠༡༢༣༤༥༦༧༨༩"
    sides = "ནབ"

    result = ""
    is_first_fn_passed = False
    for line in text.split("\n"):
        if not line:
            continue
        if line.startswith("[v"):
            continue
        if ":" in line:
            if is_first_fn_passed:
                result += f"{line}\n"
            else:
                result += f"\n{line}\n"
                is_first_fn_passed = True
        elif line.startswith("[") and line[1] != "^":
            en_pg_index = line.strip()[1:-1]
            bo_pg_index = index_to_bo(en_pg_index)
            result += f"({bo_pg_index})\n"
        else:
            result += f"{line}\n"

    return result


def create_foot_notes(text_id, opf_path, metadata, index_layer):
    serializer = SerializeFootNote(
        opf_path,
        text_id=text_id,
        index_layer=index_layer,
        layers=["correction", "peydurma-note", "pagination"],
    )
    if not serializer.text_spans:
        error_msg = f"[not-found] {text_id} not found in derge-tengyur\n"
        print(error_msg)
        error_logs_fn.open("a").write(error_msg)
        return ""
    serializer.apply_layers()
    annotated_text, text_id = serializer.get_result()

    bo_text = to_bo_pagination(annotated_text)
    return f'{bo_text[0]}{metadata[text_id]["loc"]} {bo_text[1:]}'


def get_text_metadata(path):
    import xml.dom.minidom as md

    from pyewts import pyewts

    converter = pyewts()

    def _text(tag_name, to_bo=False, is_loc=False):
        tag = item.getElementsByTagName(tag_name)
        if not tag:
            return "no-author"
        if not tag[0].firstChild:
            if tag_name == "author":
                return "no-author"
            else:
                return ""
        value = tag[0].firstChild.nodeValue.strip()
        if to_bo:
            return converter.toUnicode(value)
        if is_loc:
            name = converter.toUnicode(value.split(",")[0])
            vol = converter.toUnicode(value.split(",")[1].split(" ")[1])
            return f"{name}། {vol}"
        else:
            return value

    metadata = defaultdict(dict)
    dom = md.parseString(path.read_text())
    for item in dom.getElementsByTagName("item"):
        metadata[_text("ref")] = {
            "title": _text("tib", to_bo=True),
            "author": _text("author", to_bo=True),
            "loc": _text("loc", is_loc=True),
        }

    return metadata


def truncate_title(title):
    if len(title) > 70:
        return title[:35] + "_" + title[-35:]
    else:
        return title


if __name__ == "__main__":
    data_path = Path("../openpecha-user")
    opf_path = data_path / ".openpecha/data/P000002/P000002.opf"
    notes_path = data_path / "data" / "2-1-a_reinserted"
    text_metadata_path = data_path / "data" / "tanjurd.xml"
    foot_notes_path = data_path / "data" / "foot-notes"
    text_output_path = foot_notes_path / "txts"
    docx_output_path = foot_notes_path / "docxs"

    text_metadata = get_text_metadata(text_metadata_path)
    index_layer = yaml.safe_load((opf_path / "index.yml").open())

    for path in sorted(notes_path.iterdir()):
        text_id = path.name.split("_")[0]
        print(f"[INFO] Processing {text_id} ...")

        text_id = "D1943"
        if text_id not in text_metadata:
            error_msg = f"[not-found] {text_id} not found in derge-tengyur\n"
            print(error_msg)
            error_logs_fn.open("a").write(error_msg)
            continue

        # create output path for txt and docx file
        text_author_path = text_output_path / text_metadata[text_id]["author"]
        text_author_path.mkdir(exist_ok=True, parents=True)
        text_title = truncate_title(text_metadata[text_id]["title"])
        text_fn = text_author_path / f"{text_id}-{text_title}.txt"

        docx_author_path = docx_output_path / text_metadata[text_id]["author"]
        docx_author_path.mkdir(exist_ok=True, parents=True)
        docx_fn = docx_author_path / f"{text_id}-{text_title}.docx"
        if text_fn.is_file() and docx_fn.is_file():
            continue

        # create generate txt file
        foot_noted_text = create_foot_notes(
            text_id, opf_path, text_metadata, index_layer
        )
        if not foot_noted_text or "[^" not in foot_noted_text:
            continue
        text_fn.write_text(foot_noted_text)

        # convert txt file to docx with pandoc
        cmd = f"pandoc +RTS -K100000000 -RTS -s -o {docx_fn} {text_fn}"
        os.system(cmd)
        if docx_fn.is_file():
            print(f"[INFO] Generated {text_id} docx")



import shutil
from pathlib import Path

import pytest
import yaml

from openpecha.blupdate import Blupdate, PechaBaseUpdate
from openpecha.utils import load_yaml


@pytest.fixture(params=[{"srcbl": "abefghijkl", "dstbl": "abcdefgkl"}])
def inputs(request):
    return request.param


@pytest.fixture(params=[{"expected_result": [(0, 2, 0), (2, 5, 2), (8, 10, -1)]}])
def compute_cctv_test_cases(request):
    return request.param


def test_compute_cctv(inputs, compute_cctv_test_cases):
    updater = Blupdate(inputs["srcbl"], inputs["dstbl"])

    result = updater.cctv

    assert result == compute_cctv_test_cases["expected_result"]


@pytest.fixture(
    params=[
        {"srcblcoord": 3, "expected_result": (2, True)},
        {"srcblcoord": 7, "expected_result": (1, False)},
        {"srcblcoord": 9, "expected_result": (-1, True)},
        {"srcblcoord": 5, "expected_result": (1, False)},
    ]
)
def cctv_for_coord_test_cases(request):
    return request.param


def test_get_cctv_for_coord(inputs, cctv_for_coord_test_cases):
    updater = Blupdate(inputs["srcbl"], inputs["dstbl"])

    result = updater.get_cctv_for_coord(cctv_for_coord_test_cases["srcblcoord"])

    assert result == cctv_for_coord_test_cases["expected_result"]


@pytest.fixture(
    params=[
        {"srcblcoord": 3, "expected_result": ("abe", "fghi")},
        {"srcblcoord": 7, "expected_result": ("fghi", "jkl")},
    ]
)
def get_context_test_cases(request):
    return request.param


def test_get_context(inputs, get_context_test_cases):
    updater = Blupdate(inputs["srcbl"], inputs["dstbl"], context_len=4)

    result = updater.get_context(get_context_test_cases["srcblcoord"])

    assert result == get_context_test_cases["expected_result"]


@pytest.fixture(
    params=[
        {"context": ("fghi", "jkl"), "dstcoordestimate": 8, "expected_result": 7},
        {"context": ("ab", "efgh"), "dstcoordestimate": 4, "expected_result": 4},
        {"context": ("ghij", "kl"), "dstcoordestimate": 7, "expected_result": 7},
    ]
)
def dmp_find_test_cases(request):
    return request.param


def test_dmp_find(inputs, dmp_find_test_cases):
    updater = Blupdate(inputs["srcbl"], inputs["dstbl"], context_len=4)

    result = updater.dmp_find(
        dmp_find_test_cases["context"], dmp_find_test_cases["dstcoordestimate"]
    )

    assert result == dmp_find_test_cases["expected_result"]


@pytest.fixture(
    params=[
        {"srcblcoord": 0, "expected_result": 0},
        {"srcblcoord": 2, "expected_result": 4},
        {"srcblcoord": 7, "expected_result": 7},
    ]
)
def updated_coord(request):
    return request.param


def test_updated_coord(inputs, updated_coord):
    updater = Blupdate(inputs["srcbl"], inputs["dstbl"], context_len=4)

    result = updater.get_updated_coord(updated_coord["srcblcoord"])

    assert result == updated_coord["expected_result"]


# Test on real text
data_path = Path("tests/data/blupdate")


@pytest.fixture(scope="module")
def updater():
    srcbl = (data_path / "v1" / "v1.opf" / "base.txt").read_text()
    dstbl = (data_path / "v2" / "v2.opf" / "base.txt").read_text()
    updater = Blupdate(srcbl, dstbl)
    return updater


def get_layer(layer_name, result, expected):
    src_layer = (
        data_path / result / f"{result}.opf" / "layers" / "v001" / f"{layer_name}.yml"
    )
    dst_layer = (
        data_path
        / expected
        / f"{expected}.opf"
        / "layers"
        / "v001"
        / f"{layer_name}.yml"
    )
    return yaml.safe_load(src_layer.open()), yaml.safe_load(dst_layer.open())


def is_layer_same(result_layer, expected_layer):
    for result_ann, expected_ann in zip(
        result_layer["annotations"], expected_layer["annotations"]
    ):
        assert expected_ann["span"]["start"] == result_ann["span"]["start"]
        assert expected_ann["span"]["end"] == result_ann["span"]["end"]


def is_index_same(result, expected):
    for result_ann, expected_ann in zip(result["annotations"], expected["annotations"]):
        # assert span
        for result_span_vol, expected_span_vol in zip(
            result_ann["span"], expected_ann["span"]
        ):
            assert (
                result_span_vol["span"]["start"] == expected_span_vol["span"]["start"]
            )
            assert result_span_vol["span"]["end"] == expected_span_vol["span"]["end"]

        # assert parts
        for result_part, expected_part in zip(
            result_ann["parts"], expected_ann["parts"]
        ):
            for result_part_span_vol, expected_part_span_vol in zip(
                result_part["span"], expected_part["span"]
            ):
                assert (
                    result_part_span_vol["span"]["start"]
                    == expected_part_span_vol["span"]["start"]
                )
                assert (
                    result_part_span_vol["span"]["end"]
                    == expected_part_span_vol["span"]["end"]
                )


def test_update_index_layer():
    src_opf_path = data_path / "v1" / "v1.opf"
    dst_opf_path = data_path / "v1_base_edited" / "v1_base_edited.opf"
    expected_opf_path = data_path / "v2" / "v2.opf"

    # edit v1 base
    dst_opf_path.mkdir(exist_ok=True, parents=True)
    shutil.copytree(
        str(src_opf_path / "layers"), str(dst_opf_path / "layers"), dirs_exist_ok=True
    )
    shutil.copy(str(src_opf_path / "index.yml"), str(dst_opf_path / "index.yml"))
    shutil.copytree(
        str(expected_opf_path / "base"), str(dst_opf_path / "base"), dirs_exist_ok=True
    )

    pecha = PechaBaseUpdate(src_opf_path, dst_opf_path)
    pecha.update()

    # test annotations layers
    for layer in ["title", "yigchung", "quotes", "tsawa", "sapche"]:
        result_layer, expected_layer = get_layer(layer, "v1_base_edited", "v2")
        is_layer_same(result_layer, expected_layer)

    # test index layer
    result_index_layer = load_yaml(dst_opf_path / "index.yml")
    expected_index_layer = load_yaml(expected_opf_path / "index.yml")

    is_index_same(result_index_layer, expected_index_layer)

    shutil.rmtree(str(dst_opf_path.parent))



import pytest

from openpecha.cli import download_pecha


@pytest.mark.skip
def test_download_pecha():
    pecha_path = download_pecha("P000792")
    print(pecha_path)


if __name__ == "__main__":
    test_download_pecha()



from openpecha.formatters import HFMLFormatter

formatter_input_path = "../openpecha-user/publication/P000001-test"
formatter_output_path = "../openpecha-user/opfs"

formatter = HFMLFormatter(output_path=formatter_output_path)
formatter.create_opf(formatter_input_path)



import copy
import re
from pathlib import Path

from .formatted_dmp import FormattedDMP


class Patches:
    def __init__(self):
        self.modes = ["dergepage", "cm"]

    def clean(self, patches, mode):
        assert mode in self.modes, "The given mode is not valid. Options are: " + str(
            self.modes
        )

        if mode == "dergepage":
            return self.clean_patches(
                patches, self.is_dergepage_diff, self.find_dergepage_str
            )
        elif mode == "cm":
            return self.clean_patches(patches, self.is_cm_diff, self.find_cm_str)

    @staticmethod
    def is_dergepage_diff(diff):
        op, text = diff
        return op == 1 and "[" in text and "]" in text

    @staticmethod
    def find_dergepage_str(text):
        regex = r"(\\\[.*?\\\])"
        match = re.findall(regex, text)
        return match[0] if match else None

    @staticmethod
    def is_cm_diff(diff):
        op, text = diff
        return op == 1 and ("{" in text or "}" in text)

    @staticmethod
    def find_cm_str(text):
        # nothing to do
        return text

    @staticmethod
    def clean_patches(orig_patches, is_needed_diff, find_needed_str):
        """
        Selects and cleans a given patch list using functions given as arguments
        Intended for patches generated by Google's diff_match_patch module.

        :param orig_patches: original set of patches
        :type orig_patches: patch objects created with DMP.patch_make(str, str)
        :param is_needed_diff: test to see if a diff contains a wanted modification
        :type is_needed_diff: funct returning a boolean
        :param find_needed_str: finds the parts of the diff that are to keep
        :type find_needed_str: funct returning the replacement str
        :return: filtered set of patches that only contain the needed modifications
        """
        new_patches = []
        for patch in orig_patches:
            keep = False
            new_diffs = []
            for diff in patch.diffs:

                # find diff to modify
                if is_needed_diff(diff):
                    op, text = diff
                    needed = find_needed_str(text)
                    if needed:
                        new_diffs.append((op, needed))
                        keep = True

                # Important: keep the diffs with no modification used as context
                # (used by DMP to calculate the correct patching location)
                elif diff[0] == 0:
                    new_diffs.append(diff)

            # select only relevant patches
            if keep:
                patch.diffs = new_diffs  # replace the old diffs with the modified ones
                new_patches.append(patch)

        return new_patches

    @staticmethod
    def _deconstruct_cm_patch(patch):
        before = []
        cm = []
        after = []
        state = -1
        for diff in patch.diffs:
            string = diff[1]
            if "{" in string:
                state = 0

            if state == -1:
                before.append(diff)
            elif state == 0:
                cm.append(diff)
            elif state == 1:
                after.append(diff)
            else:
                raise ValueError("this should not happen.")

            if "}" in string:
                state = 1
        return before, cm, after

    def format_cm_operations(self, patches):
        for patch in patches:
            before, cm, after = self._deconstruct_cm_patch(patch)
            if cm:
                # process diffs
                string = "".join([c[1] for c in cm])
                op = string[1]
                string = (
                    string.replace("{", "")
                    .replace("}", "")
                    .replace("~", "")
                    .replace("+", "")
                    .replace("-", "")
                )

                if op == "+":
                    cm = [(1, string)]
                elif op == "-":
                    cm = [(-1, string)]
                elif op == "~":
                    to_del, to_add = string.split(">")
                    cm = [(-1, to_del), (1, to_add)]
                else:
                    raise ValueError("this should not happen")

                # replace old diffs with new diffs
                patch.diffs = before + cm + after

        return patches


class OpenPoti:
    def __init__(self, dirs_conf):
        self.dmp = FormattedDMP()
        self.cp = Patches()
        self.dirs = self.set_dirs(dirs_conf)
        self.current = {"name": "", "base": None, "layers": {}}

    @staticmethod
    def set_dirs(dir_paths):
        """
        populates self.dirs and creates the corresponding folder scaffolding
        :param dir_paths: paths as strings
        :return: content of self.dirs
        """
        dirs = {
            "bases": Path(dir_paths["bases"]),
            "layers": Path(dir_paths["layers"]),
            "input": Path(dir_paths["input"]),
            "output": Path(dir_paths["output"]),
        }
        dirs["layers"].mkdir(parents=True, exist_ok=True)
        return dirs

    def reset_current(self):
        self.current = {"base": None, "layers": {}}

    def load_poti(self, pch_path):
        """
        loads in self.current all the files pertaining to the current layered text
        :param pch_path: the base-name of the layered text to load
        """
        assert pch_path in [
            b.stem for b in self.dirs["bases"].glob("*.txt")
        ], f"base file missing for {pch_path}"
        assert pch_path in [
            b.name for b in self.dirs["layers"].glob("*") if b.is_dir()
        ], f"folder of layers missing for {pch_path}"

        base = self.dirs["bases"] / f"{pch_path}.txt"
        self.current["name"] = pch_path
        self.current["base"] = base.read_text(encoding="utf-8-sig")

        lyrs = self.dirs["layers"] / pch_path
        for lyr in lyrs.glob("*.*"):
            lyr_type, lyr_name = lyr.suffix[1:], lyr.stem
            if lyr_name not in self.current["layers"]:
                self.current["layers"][lyr_name] = {}
            self.current["layers"][lyr_name][lyr_type] = lyr.read_text(
                encoding="utf-8-sig"
            )

    @staticmethod
    def build_md_layers(text):
        """

        :param text: string to be analyzed
        :return: the base string and a list of name/layer tuples.
        :rtype: str, [(str, str), ...]
        """
        chunks = re.split(r"{([+\-~]{2}(?:.|\n)+?)[+\-~]{2}}", text)
        base = [chunks[i] for i in range(0, len(chunks), 2)]
        base = "".join(base)

        lyrs = {
            "book_title": [],
            "book_sub_title": [],
            "chapter_title": [],
            "sapche": [],
            "tsawa": [],
            "quotes": [],
            "yigchung": [],
        }

        mod_idx = list(range(1, len(chunks), 2))
        for num, c in enumerate(chunks):
            if num in mod_idx:
                if c.startswith("++"):
                    c = c[2:]
                    if c == "*":  # test for yigchung
                        lyrs["yigchung"].append(c)
                    elif c == "**":
                        lyrs["tsawa"].append(c)
                    elif c == "~~":
                        lyrs["quotes"].append(c)
                    elif c == "[" or c == "]":
                        lyrs["sapche"].append(c)
                    elif c == "###":
                        lyrs["chapter_title"].append(c)
                    elif c == "##":
                        lyrs["book_sub_title"].append(c)
                    elif c == "#":
                        lyrs["book_title"].append(c)
                    elif c.startswith("="):  # test for titles (to be improved)
                        lyrs["book_title"].append(c)
                else:
                    raise ValueError(
                        "This should not happen: the file used to create an openpecha is"
                    )
            else:
                for k in lyrs.keys():
                    lyrs[k].append(c)

        return base, [(k, "".join(v)) for k, v in lyrs.items()]

    def new_poti(self, basename):
        """
        Create the scafolding for a new layered text:
            - <basename>.txt        the base layer
            - layers/<basename>/    the layer's folder
        """
        basename = self.dirs["input"] / basename
        assert (
            basename.is_file() and basename.suffix == ".txt"
        ), f'{basename} is not found in {self.dirs["input"]}.'
        dump = basename.read_text(encoding="utf-8-sig")

        # separate base from title and yigchung
        base, md_lyrs = self.build_md_layers(dump)

        # write base
        Path(self.dirs["bases"] / "base.txt").write_text(base, encoding="utf-8-sig")

        # write layers
        # lyr_dir = Path(self.dirs['layers'] / basename.stem)
        # lyr_dir.mkdir(exist_ok=True)

        # self.load_poti(basename.stem)
        for name, lyr in md_lyrs:
            self.create_layer(base, lyr, name, clean_patch=False)

        # layers info
        ((self.dirs["layers"].parent) / "layers.yml").write_text(
            "layers:\n  - book_sub_title:\n  - book_title:\n  - chapter_title:\n  - quotes:\n  - sapche:\n  - tsawa:"
        )

    def create_layer(self, orig, mod, name, deps="", clean_patch=True):
        """
        TODO: should not need orig argument. should build against the base in self.current
        creates and writes to file a new layer
        :param orig: the original text
        :type orig: str
        :param mod: the modified text
                    (contains the modifications to be turned into a layer)
        :type mod: str
        :param name: the name of the new layer
        :type name: str
        :param deps: the dependency of the new layer. layers are separated by "\n"s
        :param clean_patch:
        """
        lyr = self.dmp.patch_make(orig, mod)
        lyr = self.cp.clean(lyr, "cm") if clean_patch else lyr
        lyr = self.cp.format_cm_operations(lyr)
        lyr = "\n".join([self.dmp.decode_patch(str(p)) for p in lyr])

        if lyr:
            lyr_file = self.dirs["layers"] / self.current["name"] / f"{name}.layer"
            lyr_file.write_text(lyr, encoding="utf-8-sig")

            if deps:
                dep_file = self.dirs["layers"] / self.current["name"] / f"{name}.deps"
                dep_file.write_text(deps, encoding="utf-8-sig")

    @staticmethod
    def _format_notes(patches):
        footnotes = ""
        for num, patch in enumerate(patches):
            diffs = patch.diffs
            assert len(diffs) == 3 or len(diffs) == 4, "unexpected diff"
            mod = diffs[1:-1]
            p_type = [d[0] for d in mod]
            strings = [d[1] for d in mod]
            if p_type == [-1, 1]:
                note = ">>".join(strings)
            elif p_type == [1]:
                note = "+" + "".join(strings)

            elif p_type == [-1]:
                note = "-" + "".join(strings)
            else:
                raise ValueError("this should not happen")
            if note:
                diffs.insert(-1, (1, f"[^{num+1}]"))
                footnotes += f"\n[^{num+1}]: " + note

        return footnotes, patches

    def create_view(self, layers, mode=None, cor_as_note=True):
        view = copy.deepcopy(self.current["base"])
        fails = {}
        footnotes = ""

        for lyr_name in layers:
            lyr_str = self.current["layers"][lyr_name]["layer"]
            lyr = self.dmp.patch_fromText(lyr_str)
            if cor_as_note and lyr_name == "correction":
                footnotes, lyr = self._format_notes(lyr)

            view, res = self.dmp.patch_apply(lyr, view, mode)
            mstk = {}
            for num, r in enumerate(res):
                if not r:
                    mstk[str(num + 1)] = self.dmp.format_patch(lyr[num])
            if mstk:
                fails[lyr_name] = mstk

        footnotes = "\n\n" + footnotes if footnotes else footnotes
        return view + footnotes, fails

    def write_views(self, layers, view_type):
        self.load_poti(self.current["name"])
        if view_type == "export":
            view, fails = self.create_view(layers)
        elif view_type == "edit":
            view, fails = self.create_view(layers, mode="CM")
        else:
            raise ValueError("this should not happen.")

        out = (
            self.dirs["output"]
            / f'{self.current["name"]}_{view_type}_{"+".join(layers)}.txt'
        )
        out.write_text(view, encoding="utf-8-sig")
        if fails:
            out = (
                self.dirs["output"]
                / f'{self.current["name"]}_export_{"+".join(layers)}_mistakes.txt'
            )
            out.write_text(str(fails), encoding="utf-8-sig")


if __name__ == "__main__":
    # basic usecase
    conf = {
        "bases": "plaintext",
        "layers": "admin/layers",
        "input": "user/edited",
        "output": "user/output",
    }

    existing = OpenPoti(conf)
    existing.new_poti("test_base+md.txt")

    existing.create_layer(
        existing.current["base"],
        Path(existing.dirs["input"] / "text_base+md_correction.txt").read_text(
            encoding="utf-8-sig"
        ),
        "correction",
    )

    to_apply = ["yigchung", "title1", "correction"]
    existing.write_views(to_apply, "edit")
    existing.write_views(to_apply, "export")



from openpecha.core.layer import InitialCreationEnum, Layer, LayerEnum, MetaData
from openpecha.core.pecha import OpenPecha


def test_init_pecha():
    openpecha = OpenPecha(
        base={"v001": "this is base"},
        layers={
            "v001": {
                LayerEnum.citation: Layer(
                    annotation_type=LayerEnum.citation, revision="00001", annotations={}
                )
            }
        },
        meta=MetaData(id="P000001", initial_creation_type=InitialCreationEnum.ebook),
    )
    assert openpecha.meta
    assert openpecha.get_base("v001")
    assert openpecha.get_layer("v001", LayerEnum.citation)
    assert openpecha.save("output")



import argparse
from pathlib import Path

import yaml


def get_pg_layer(path):
    pecha_path = Path(path)
    pecha_id = pecha_path.name
    layer_path = pecha_path / f"{pecha_id}.opf" / "layers"
    for vol_path in layer_path.iterdir():
        print(f"[INFO] Processing {vol_path} ...")
        pg_fn = vol_path / "pagination.yml"
        yield pg_fn


def remove_lines(fn):
    ann = yaml.safe_load(fn.open())
    pg_ann = []
    for a in ann["content"]:
        if a["type"] == "page":
            pg_ann.append(a)
    ann["content"] = pg_ann
    yaml.dump(ann, fn.open("w"), default_flow_style=False, sort_keys=False)


if __name__ == "__main__":
    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument(
        "--path", "-p", type=str, help="directory path containing all the data"
    )
    args = ap.parse_args()

    for pg_fn in get_pg_layer(args.path):
        remove_lines(pg_fn)



import os
import platform
import struct
from pathlib import Path

M_TYPE = platform.system()
GIT_URL = {
    "Darwin-64": "https://sourceforge.net/projects/git-osx-installer/files/git-2.23.0-intel-universal-mavericks.dmg",
    "Darwin-32": "https://sourceforge.net/projects/git-osx-installer/files/git-2.23.0-intel-universal-mavericks.dmg",
    "Windows-32": "https://github.com/git-for-windows/git/releases/download/v2.25.0.windows.1/Git-2.25.0-32-bit.exe",
    "Windows-64": "https://github.com/git-for-windows/git/releases/download/v2.25.0.windows.1/Git-2.25.0-64-bit.exe",
}


def run_cmd(cmd):
    os.system(cmd)


def install_dependencies():
    dependencies = ["requests", "tqdm"]
    for dependency in dependencies:
        run_cmd(f"pip3 install {dependency}")


def git_pkg_fn():
    mount_path = Path("/Volumes")
    git_pkg_dir = list(mount_path.glob("Git*"))[0]
    git_pkg_path = str(list(git_pkg_dir.glob("git*"))[0])
    return git_pkg_path.replace(" ", r"\ ")


def is_git_installed():
    from shutil import which

    return which("git") is not None


def download_git(url):
    import requests
    from tqdm import tqdm

    git_installer_fn = Path.home() / "Downloads" / url.split("/")[-1]
    if git_installer_fn.is_file():
        print("[INFO] Git installer already downloaded")
        return git_installer_fn

    r = requests.get(url, stream=True)
    total_size = int(r.headers.get("content-length", 0))
    block_size = 1024  # 1 kibibyte
    t = tqdm(total=total_size, unit="iB", unit_scale=True)

    with git_installer_fn.open("wb") as f:
        for data in r.iter_content(block_size):
            t.update(len(data))
            f.write(data)
    t.close()

    return git_installer_fn


def install_git():
    bit = sys_32_or_64()
    if bit == 32:
        git_installer_fn = download_git(GIT_URL[f"{M_TYPE}-32"])
    else:
        git_installer_fn = download_git(GIT_URL[f"{M_TYPE}-64"])

    if M_TYPE == "Darwin":
        # mount and install git
        run_cmd(f"hdiutil attach {git_installer_fn}")
        run_cmd(f"sudo installer -pkg {git_pkg_fn()} -target /")
    elif M_TYPE == "Windows":
        run_cmd(f"start {git_installer_fn}")
    else:
        pass


def sys_32_or_64():
    return struct.calcsize("P") * 8


def install_op_toolkit():
    run_cmd("pip3 install openpecha")


def install():
    print("[INFO] Installing dependencies ...")
    install_dependencies()

    print("[INFO] Installing Git ...")
    if is_git_installed():
        print("[INFO] Git already installed")
    else:
        install_git()

    print("[INFO] Installing OpenPecha ToolKit ....")
    install_op_toolkit()


if __name__ == "__main__":
    install()



import argparse
import csv
import json
import os
from pathlib import Path

import yaml

from openpecha.github_utils import github_publish

START_PECHA = 10
catalog_data_path = Path("../openpoti-catalog/data/")
web_meta_path = (
    catalog_data_path / "website_metadata.json"
)  # save metadata in .opf/meta.yml
ebook_meta_path = (
    catalog_data_path / "ebook_metadata.csv"
)  # replace ID with new Pecha ID

# load the catalog data
web_meta = json.load(web_meta_path.open())
ebook_meta = csv.reader(ebook_meta_path.open())
ebook_meta_updated = [["ID", "Book Title", "Vol Numner", "Book Author", "Filename"]]


def format_ebooks(path):
    global START_PECHA
    ebook_path = path / "OEBPS"
    cmd = f"openpecha format -n tsadra -i {START_PECHA} {str(ebook_path)}"
    os.system(cmd)
    pecha_id = f"P{START_PECHA:06}"
    if not Path(f"./opfs/{pecha_id}/{pecha_id}.opf/meta.yml").is_file():
        raise Exception
    START_PECHA += 1


def create_readme(metadata, path):
    result = ""
    web_metadata = metadata["web_metadata"]
    # add title
    title = web_metadata["title"]
    result += f"## Title\n\t- {title['bo']}\n\t- {title['en']}\n\n"

    # add author
    author = web_metadata["author"]
    result += f"## Author\n\t- {author['bo']}\n\t- {author['en']}\n\n"

    readme_fn = path / "README.md"
    readme_fn.write_text(result)


def create_meta_fn(path):
    # populate web_metadata in opf meta.yml
    opf_meta_fn = path / f"{path.name}.opf" / "meta.yml"
    opf_meta = yaml.safe_load(opf_meta_fn.open())
    sku = opf_meta["ebook_metadata"]["sku"]
    for o in web_meta:
        if o["sku"] == sku:
            opf_meta["web_metadata"] = o
            yaml.dump(
                opf_meta,
                opf_meta_fn.open("w"),
                default_flow_style=False,
                sort_keys=False,
                allow_unicode=True,
            )
            create_readme(opf_meta, path)
            break

    for row in ebook_meta:
        if row[-1] == sku:
            row[0] = path.name
            ebook_meta_updated.append(row)
            break


if __name__ == "__main__":
    # ap = argparse.ArgumentParser(add_help=False)
    # ap.add_argument("--input", "-i", type=str, help="path to tsadra ebooks")
    # ap.add_argument("--output", "-o", type=str, help="path to opfs")
    # args = ap.parse_args()

    input = "./bo_crawler/bo_crawler/data/tsadra/data/ebooks/"
    output = "./opfs"

    errors = ""
    errors_fn = Path("errors.txt")

    # format all the ebooks to opf
    print("[INFO] OPF Formating ...")
    for path in Path(input).iterdir():
        if path.suffix == ".epub":
            continue
        print(f"\t- formatting {path.name}")
        try:
            format_ebooks(path)
        except Exception:
            errors += path.name + "\n"

    # publish all the opfs to github
    print("[INFO] OpenPecha Publish ...")
    for path in Path(output).iterdir():
        try:
            create_meta_fn(path)
        except Exception:
            errors += path.name + "\n"
        # print(f'\t- Publishing {path.name} ...')
        # github_publish(path)

    errors_fn.write_text(errors)

    # save ebook_metadata
    with ebook_meta_path.open("w") as f:
        csv_writer = csv.writer(f, delimiter=",")
        for row in ebook_meta_updated:
            csv_writer.writerows(row)



from pathlib import Path

from openpecha.serializers import SerializeHFML


def serialize_vol(opf_path, vol_id, out_path):
    serializer = SerializeHFML(opf_path, vol_id=vol_id)
    serializer.apply_layers()

    result = serializer.get_result()
    out_fn = out_path / f"{vol_id}.txt"
    out_fn.write_text(result)


if __name__ == "__main__":
    data_path = Path("../openpecha-user")
    opf_path = data_path / ".openpecha/data/P000005/P000005.opf"
    out_path = data_path / "text"
    out_path.mkdir(exist_ok=True, parents=True)

    for fn in (opf_path / "base").iterdir():
        # if not fn.stem == 'v014': continue
        print(f"[INFO] Processing {fn.stem} ....")
        serialize_vol(opf_path, fn.stem, out_path)



import argparse
import shutil
from pathlib import Path


def get_vol_id(path):
    vol_num = int(path.stem[1:])
    return f"v{vol_num:03}"


def rename_in_base(opf_path):
    base_path = opf_path / "base"
    for fn in base_path.iterdir():
        target_fn = base_path / f"{get_vol_id(fn)}.txt"
        fn.rename(target_fn)


def rename_in_layer(opf_path):
    layer_path = opf_path / "layers"
    for vol_dir in layer_path.iterdir():
        target_vol_dir = layer_path / get_vol_id(vol_dir)
        shutil.move(str(vol_dir), str(target_vol_dir))


if __name__ == "__main__":
    ap = argparse.ArgumentParser(add_help=False)
    ap.add_argument(
        "--path", "-p", type=str, help="directory path containing all the data"
    )
    args = ap.parse_args()

    pecha_path = Path(args.path)
    pecha_opf = pecha_path / f"{pecha_path.name}.opf"
    rename_in_base(pecha_opf)
    rename_in_layer(pecha_opf)



import re
from collections import defaultdict
from copy import deepcopy
from pathlib import Path

import diff_match_patch as dmp_module
import yaml

from openpecha.blupdate import Blupdate
from openpecha.formatters import BaseFormatter
from openpecha.formatters.format import Layer, Peydurma

error_log_fn = Path("./error_logs") / Path(__file__).stem
if error_log_fn.is_file():
    error_log_fn.unlink()


def get_notes(fn):
    raw_text = fn.read_text()
    if "[^" not in raw_text:
        return None, None
    pattern = r""
    if "[^1K]" in raw_text:
        pattern = r"\[\^1K\]"
    else:
        pattern = r"\[\^2K\]"
    foot_note_idx = max(re.finditer(pattern, raw_text), key=lambda x: x.start(0)).start(
        0
    )

    # preprocess text
    text = raw_text[: foot_note_idx - 1].replace("\ufeff", "")
    text = text[1:] if text[0] == "a" else text
    text = text.replace("\n", " ")
    text = text.replace("a", "\n")

    is_note = False
    base_text_len = 0
    text_notes = []
    base_text = ""
    for i, c in enumerate(text):
        if c == "[" and text[i + 1] == "^":
            is_note = True
        elif is_note and c == "]":
            is_note = False
            text_notes.append(base_text_len - 1)
        elif not is_note:
            base_text_len += 1
            base_text += c

    foot_notes = [
        foot_note.split(":")[1].strip()
        for foot_note in raw_text[foot_note_idx:].split("\n")
        if foot_note
    ]

    return zip(text_notes, foot_notes), base_text.strip()


def get_index(path):
    return yaml.safe_load((path / "index.yml").open())


def get_layer_ann(path, vol, layer_name):
    return yaml.safe_load((path / "layers" / vol / f"{layer_name}.yml").open())


def save_layer_ann(layer_ann, path, vol, layer_name):
    return yaml.safe_dump(
        layer_ann,
        (path / "layers" / vol / f"{layer_name}.yml").open("w"),
        default_flow_style=False,
        sort_keys=False,
        allow_unicode=True,
    )


def create_layer(layer_name):
    layer = deepcopy(Layer)
    layer["id"] = BaseFormatter().get_unique_id()
    layer["annotation_type"] = layer_name
    layer["revision"] = f"{1:05}"
    return layer


def get_work_text(path, vol):
    span = vol["span"]
    return (path / f"{vol['vol']}.txt").read_text()[span["start"] : span["end"]]


def update_layer(note, bl, note_idx, vol, layer_ann):
    dst_note_idx = bl.get_updated_coord(note[0])
    if dst_note_idx == -1:
        return
    vol_note_idx = dst_note_idx + vol["span"]["start"]

    if layer_ann["annotations"] and len(layer_ann["annotations"]) > note_idx + 1:
        ann = layer_ann["annotations"][0]
        ann["note"] = note[1]
        ann["span"] = vol_note_idx
        ann["span"] = vol_note_idx
    else:
        peydurma = deepcopy(Peydurma)
        peydurma["id"] = BaseFormatter().get_unique_id()
        peydurma["note"] = note[1]
        peydurma["span"]["start"] = vol_note_idx
        peydurma["span"]["end"] = vol_note_idx
        layer_ann["annotations"].append(peydurma)


def reinsert(pecha_path, notes_path, layer_name):
    pecha_opf_path = pecha_path / f"{pecha_path.name}.opf"
    pecha_index = get_index(pecha_opf_path)

    layer_ann = None
    prev_vol = None
    note_idx = 0
    for text_ann in pecha_index["annotations"]:
        notes_fn = notes_path / f'{text_ann["work"]}_a_reinserted.txt'
        if not notes_fn.is_file():
            continue
        # if not text_ann['work'] == 'D1115': continue
        print(f'[INFO] Processing {text_ann["work"]} ...')
        notes, base_text = get_notes(notes_fn)
        if not notes:
            continue

        # cross-vol text have mutlitple layer files in terms of volume
        for vol in text_ann["span"]:
            work_text = get_work_text(pecha_opf_path, vol)
            if prev_vol != vol["vol"]:
                # first save the layer of previous volume.
                if layer_ann:
                    save_layer_ann(layer_ann, pecha_opf_path, prev_vol[5:], layer_name)

                # then load the layer of next volume
                if layer_name in ["peydurma"]:
                    layer_ann = get_layer_ann(
                        pecha_opf_path, vol["vol"][5:], layer_name
                    )
                else:
                    layer_ann = create_layer(layer_name)

                note_idx = 0

            # insert each foot_note of current text into layer with note_idx.
            bl = Blupdate(base_text, work_text)
            if len(bl.cctv) <= 1:
                error_msg = f'[diff-text] {text_ann["work"]} (derge-tengyur)\n'
                print(error_msg)
                error_log_fn.open("a").write(error_msg)
                break
            for note in notes:
                update_layer(note, bl, note_idx, vol, layer_ann)
                note_idx += 1

            prev_vol = vol["vol"]


if __name__ == "__main__":
    DATA_PATH = Path("../openpecha-user")
    OPF_PECHA_PATH = DATA_PATH / ".openpecha/data/P000002"
    NOTES_PATH = DATA_PATH / "data" / "2-1-a_reinserted"

    reinsert(OPF_PECHA_PATH, NOTES_PATH, "peydurma-note")



# -*- coding: utf-8 -*-

# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: https://doc.scrapy.org/en/latest/topics/item-pipeline.html


# Spiders
from bo_crawler.spiders.tsadra_spider import TsadraSpider


class BoCrawlerPipeline(object):
    def process_item(self, item, spider):
        metadata = {}
        if spider.name == TsadraSpider.name:
            metadata = {
                "title": {"en": item["title_en"], "bo": item["title_bo"]},
                "collection": {
                    "en": item["collection_en"],
                    "bo": item["collection_bo"],
                },
                "author": {"en": item["author_en"], "bo": item["author_bo"]},
                "pusblisher": {"en": item["publisher_en"], "bo": item["publisher_bo"]},
                "sku": item["sku"],
                "category": {"en": item["category_en"], "bo": item["category_bo"]},
                "org": {"bo": item["org_en"], "en": item["org_bo"]},
                "website": item["website"],
                "aquisition_data": item["aquisition_data"],
                "filename": item["filename"],
                "description": {
                    "en": item["description_en"],
                    "bo": item["description_bo"],
                },
            }

        return metadata



from uuid import uuid4

from openpecha.formatters.layers import *
from openpecha.formatters.layers import AnnType, Span, _attr_names


def test_span():
    start, end = 100, 200
    span = Span(start, end)

    result = span

    expected = {_attr_names.START: start, _attr_names.END: end}
    assert expected == result


def get_fake_pg():
    return 100, 200, "1a", "first page", "1000.png"


def test_ann():
    start, end, pg_index, pg_info, pg_ref = get_fake_pg()
    span = Span(start, end)
    page = Page(span, page_index=pg_index, page_info=pg_info, page_ref=pg_ref)

    result = page

    expected = {
        _attr_names.PAGE_INDEX: pg_index,
        _attr_names.PAGE_INFO: pg_info,
        _attr_names.PAGE_REFERENCE: pg_ref,
        _attr_names.SPAN: {_attr_names.START: start, _attr_names.END: end},
    }

    assert expected == result


def test_layer():
    # create Page layer
    layer_id = uuid4().hex
    ann_type = AnnType.pagination
    rev = f"{1:05}"
    Pagination_layer = Layer(layer_id, ann_type, rev=rev)
    start, end, pg_index, pg_info, pg_ref = get_fake_pg()

    # create page annotaion
    span = Span(start, end)
    page = Page(span, page_index=pg_index, page_info=pg_info, page_ref=pg_ref)
    pg_id = uuid4().hex
    Pagination_layer[_attr_names.ANNOTATION][pg_id] = page

    result = Pagination_layer

    expected = {
        _attr_names.ID: layer_id,
        _attr_names.ANNOTATION_TYPE: ann_type,
        _attr_names.REVISION: rev,
        _attr_names.ANNOTATION: {
            pg_id: {
                _attr_names.PAGE_INDEX: pg_index,
                _attr_names.PAGE_INFO: pg_info,
                _attr_names.PAGE_REFERENCE: pg_ref,
                _attr_names.SPAN: {_attr_names.START: start, _attr_names.END: end},
            }
        },
    }

    assert expected == result



import os
from pathlib import Path

from openpecha.formatters import HFMLFormatter
from openpecha.serializers import HFMLSerializer

if __name__ == "__main__":
    hfml_text = "./output/P000008_hfml/P000008"
    opfs_path = Path("./output/")
    opf_path = "./output/tengyur/P000792.opf/"
    hfml_path = "./output/tengyur/pedurma/"
    pecha_id = 1

    # Converts HFML to OPF
    # formatter = HFMLFormatter(output_path=opfs_path)
    # formatter.create_opf(hfml_text, pecha_id)

    # Converts OPF to HFML
    # text_list = Path("./output/tengyur/text_list.txt").read_text()
    # texts = text_list.splitlines()
    # for text in texts:
    serializer = HFMLSerializer(opf_path, text_id="D1118")
    serializer.serialize(output_path=hfml_path)
